import sys
sys.path.insert(0, '../..')
sys.path.insert(0, '../../shared/netcode')

import queue
import grpc
from shared.netcode.netcode_pb2 import *
from shared.netcode.netcode_pb2_grpc import NotifierCommunicationStub

usage = "Usage:\n- Sender:\n  ./client.py 1\n- Receiver:\n  ./client.py 0"


def sendMessage(stub, msg):
    request = Message(content=msg)
    response = stub.SendMessage(request)


if __name__ == "__main__":
    # Check enough args
    if len(sys.argv) < 2:
        print("Error: Not enough arguments.")
        print(usage)
        exit(1)

    ########################################################################
    # SETTINGS
    ########################################################################
    is_sender = sys.argv[1]
    ########################################################################

    # print info
    print("-- SENDER --" if is_sender else "-- RECEIVER --")

    with grpc.insecure_channel('localhost:8080') as channel:
        try:
            grpc.channel_ready_future(channel).result(timeout=5)
        except grpc.FutureTimeoutError:
            print("Error: Connection to server failed.")
            exit(1)

        stub = NotifierCommunicationStub(channel)

        if is_sender:
            response = stub.Open(OpenRequest(id_code=1))
            print(response)

            sendQueue = queue.SimpleQueue()
            response_future = stub.SendMessage.future(iter(sendQueue.get, None))
            sendQueue.put_nowait(Message(content="init test"))

            print("Starting Prompt:")
            while True:
                text = input()
                if text == "":
                    print("Stopping...")
                    break
                print("Sending:", text)
                sendQueue.put_nowait(Message(content=text))

        else:
            response = stub.Answer(AnswerRequest(challenge=1))
            print(response)

            for message in stub.ReceiveMessage(Nothing()):
                print(message.content)
